#include "SceneEntity.h"
#include "glInterface.h"
#include "Geometry.h"
#include "Material.h"


namespace CHCDemo 
{


SceneEntity::SceneEntity(Geometry *geometry, 
						 Material *mat,
						 Matrix4x4 *trafo):
mGeometry(geometry), mMaterial(mat), mTransform(trafo)
{
}


void SceneEntity::Render(RenderState *state)
{
	if (!state->mTexturesEnabled && mGeometry->HasTexture())
	{
		glEnable(GL_TEXTURE_2D);
		glEnableClientState(GL_TEXTURE_COORD_ARRAY);
		state->mTexturesEnabled = true;
	}
	else if (state->mTexturesEnabled && !mGeometry->HasTexture())
	{
		glDisable(GL_TEXTURE_2D);
		glDisableClientState(GL_TEXTURE_COORD_ARRAY);
		state->mTexturesEnabled = false;
	}

	if (mMaterial) mMaterial->Render();

	if (mTransform)
	{
		glPushMatrix();
		glMultMatrixf((float *)mTransform->x);
		mGeometry->Render();

		glPopMatrix();
	}
	else
	{
		mGeometry->Render();
	}
}


void SceneEntity::SetGeometry(Geometry *geom)
{
	mGeometry = geom;
}


void SceneEntity::SetTransformation(Matrix4x4 *trafo)
{
	mTransform = trafo;
}


void SceneEntity::SetMaterial(Material *mat)
{
	mMaterial = mat;
}


void SceneEntity::SetLastRendered(int lastRendered)
{
	mLastRendered = lastRendered;
}


int SceneEntity::GetLastRendered() const
{
	return mLastRendered;
}


AxisAlignedBox3 SceneEntity::GetBoundingBox() const
{
	AxisAlignedBox3 box = mGeometry->GetBoundingBox();
	if (mTransform) Transform(box, *mTransform);

	return box;
}


}